@@ -53,7 +53,7 @@ __monitor__ = __name__
 define_config_option(
     __monitor__,
     "journal_path",
-    "Optional (defaults to /var/log/journal). Location on the filesystem of the journald logs.",
+    "Optional (defaults to `/var/log/journal`). Location of the journald logs in the filesystem.",
     convert_to=six.text_type,
     default="/var/log/journal",
 )
@@ -61,8 +61,7 @@ define_config_option(
 define_config_option(
     __monitor__,
     "journal_poll_interval",
-    "Optional (defaults to 5). The number of seconds to wait for data while polling the journal file. "
-    "Fractional values are supported. Note: This values overrides the sample_interval of the monitor",
+    "Optional (defaults to `5`). The number of seconds to wait for data while polling the journal file. Fractional values are supported. Note: This value overrides the `sample_interval` of the monitor.",
     convert_to=float,
     default=5,
 )
@@ -70,43 +69,27 @@ define_config_option(
 define_config_option(
     __monitor__,
     "journal_fields",
-    "Optional dict containing a list of journal fields to include with each message, as well as a field name to map them to.\n"
-    "Note: Not all fields need to exist in every message and only fields that exist will be included.\n"
-    "Defaults to {:\n"
-    '  "_SYSTEMD_UNIT": "unit"\n'
-    '  "_PID": "pid"\n'
-    '  "_MACHINE_ID": "machine_id"\n'
-    '  "_BOOT_ID": "boot_id"\n'
-    '  "_SOURCE_REALTIME_TIMESTAMP": timestamp"\n'
+    'Optional. A dict of journal fields to upload with each message, and a field name to map them to. Note: Not all fields need exist in every message, and only fields that exist will be included. Defaults to { "_SYSTEMD_UNIT": "unit", "_PID": "pid", "_MACHINE_ID": "machine_id", "_BOOT_ID": "boot_id", "_SOURCE_REALTIME_TIMESTAMP": "timestamp" }',
     # Note, we dont' include the _HOSTNAME field as this is likely already specified by the serverHost variable
     # and so there's no need to duplicate this information.  If needed, this can be manually specified in the
     # plugin configuration.
-    "}\n",
     default=None,
 )
 
 define_config_option(
     __monitor__,
     "journal_matches",
-    "Optional list containing 'match' strings for filtering entries."
-    'A match string follows the pattern  "FIELD=value" where FIELD is a field of '
-    'the journal entry e.g. _SYSTEMD_UNIT, _HOSTNAME, _GID and "value" is the value '
-    'to filter that field on, so a match string equal to "_SYSTEMD_UNIT=ssh.service" '
-    "would filter query results to make sure that all entries entries originated from "
-    "the `ssh.service` system unit. "
-    "The journald monitor calls the journal reader method `add_match` for each string in this list. "
-    "See the journald documentation for details on how the filtering works: "
-    "https://www.freedesktop.org/software/systemd/python-systemd/journal.html#systemd.journal.Reader.add_match "
-    "If this config item is empty or None then no filtering occurs.",
+    'Optional. A list of "match strings" to filter entries on. A match string follows the pattern  "FIELD=value", where FIELD is a field of the journal entry, e.g. `_SYSTEMD_UNIT` or `_HOSTNAME`, and "value" is the value to filter that field on. For example, "_SYSTEMD_UNIT=ssh.service" only imports entries from the `ssh.service` system unit. The journald monitor calls the journal reader method `add_match` for each string in this list. See the [add_match documentation](https://www.freedesktop.org/software/systemd/python-systemd/journal.html#systemd.journal) for more information. If this property is empty or None, then no filtering occurs.',
     default=None,
 )
 
 define_config_option(
     __monitor__,
     "id",
-    "Optional id used to differentiate between multiple journald monitors. "
-    "This is useful for configurations that define multiple journald monitors and that want to save unique checkpoints for each "
-    "monitor.  If specified, the id is also sent to the server along with other attributes under the `monitor_id` field",
+    "An optional id for the monitor. Shows in the UI as the `monitor_id` field. "
+    "Useful if you are running multiple instances of this plugin, "
+    "and you want to save unique checkpoints for each monitor. You can add "
+    "a separate `{...}` stanza for each instance the configuration file (`/etc/scalyr-agent-2/agent.json`).",
     convert_to=six.text_type,
     default="",
 )
@@ -114,7 +97,7 @@ define_config_option(
 define_config_option(
     __monitor__,
     "staleness_threshold_secs",
-    "When loading the journal events from a checkpoint, if the logs are older than this threshold, then skip to the end.",
+    "Optional (defaults to `600` seconds). When loading the journal events from a checkpoint, if the logs are older than this threshold, then skip to the end.",
     convert_to=int,
     default=10 * 60,
 )
@@ -122,7 +105,7 @@ define_config_option(
 define_config_option(
     __monitor__,
     "max_log_rotations",
-    "How many rotated logs to keep before deleting them, when writing journal entries to a log for sending to Scalyr.",
+    "Optional (defaults to `2`). How many rotated logs to keep before deleting them, when writing journal entries to a log for import.",
     convert_to=int,
     default=2,
 )
@@ -130,7 +113,7 @@ define_config_option(
 define_config_option(
     __monitor__,
     "max_log_size",
-    "Max size of a log file before we rotate it out, when writing journal entries to a log for sending to Scalyr.",
+    "Optional (defaults to `20MB`). Max size of a log file before we rotate it out, when writing journal entries to a log for import.",
     convert_to=int,
     default=20 * 1024 * 1024,
 )
@@ -259,42 +242,42 @@ class Checkpoint(object):
 class JournaldMonitor(ScalyrMonitor):  # pylint: disable=monitor-not-included-for-win32
     # fmt: off
     """
-# Journald Monitor
+# Journald
 
-A Scalyr agent monitor that imports log entries from journald.
+Import log entries from journald.
 
-The journald monitor polls systemd journal files every ``journal_poll_interval`` seconds
-and uploads any new entries to the Scalyr servers.
+An [Agent Plugin](https://app.scalyr.com/help/scalyr-agent#plugins) is a component of the Scalyr Agent, enabling the collection of more data. The source code for each plugin is available on [Github](https://github.com/scalyr/scalyr-agent-2/tree/master/scalyr_agent/builtin_monitors).
 
-@class=bg-warning docInfoPanel: An *agent monitor plugin* is a component of the Scalyr Agent. To use a plugin,
-simply add it to the ``monitors`` section of the Scalyr Agent configuration file (``/etc/scalyr/agent.json``).
-For more information, see [Agent Plugins](/help/scalyr-agent#plugins).
 
-By default, the journald monitor logs all log entries, but it can also be configured to filter entries by specific
-fields.
+## Installation
 
-## Dependencies
+1\\. Install the Scalyr Agent
 
-The journald monitor has a dependency on the Python systemd library, which is a Python wrapper around the systemd C API.
-You need to ensure this library has been installed on your system in order to use this monitor, otherwise a warning
-message will be printed if the Scalyr Agent is configured to use the journald monitor but the systemd library cannot be
-found.
+If you haven't already done so, install the [Scalyr Agent](https://app.scalyr.com/help/welcome) on servers running the journald daemon.
 
-You can install the systemd Python library via package manager e.g.:
 
-    apt-get install python-systemd  (debian/ubuntu)
-    dnf install python-systemd  (CentOS/rhel/Fedora)
+2\\. Install the `python-systemd` dependency
 
-Or install it from source using pip e.g.
+The `python-systemd` package is a wrapper for the `systemd` C API. See [https://github.com/systemd/python-systemd/](https://github.com/systemd/python-systemd/) for more information. If the Scalyr Agent cannot find this library, a warning message will show.
+
+To install `python-systemd` on Debian or Ubuntu:
+
+    apt-get install python-systemd
+
+For CentOS/rhel/Fedora:
+
+    dnf install python-systemd
+
+You can also install from source with pip:
 
     pip install systemd-python
 
-See here for more information: https://github.com/systemd/python-systemd/
 
-## Sample Configuration
+3\\. Configure the Scalyr Agent to import the journal file
 
-The following example will configure the agent to query the journal entries
-located in /var/log/journal (the default location for persisted journald logs)
+Open the Scalyr Agent configuration file, found at `/etc/scalyr-agent-2/agent.json`.
+
+Find the `monitors: [ ... ]` section, add a `{...}` stanza, and set the `module` property for journald.
 
     monitors: [
       {
@@ -302,8 +285,9 @@ located in /var/log/journal (the default location for persisted journald logs)
       }
     ]
 
-Here is an example that queries journal entries from volatile/non-persisted journals, and filters those entries to only
-include ones that originate from the ssh service
+This plugin creates a polling object (https://docs.python.org/2/library/select.html#poll-objects). A `poll` runs with a 0 second timeout - it never blocks. After processing new events, the thread sleeps for a period of time. By default, the journal file is polled for new logs every 5 seconds.
+
+If your journal file is not at the default `/var/log/journal`, add the `journal_path` property to the `{...}` stanza, and set it. Here is an example that polls journal entries from volatile/non-persisted journals:
 
     monitors: [
       {
@@ -313,13 +297,163 @@ include ones that originate from the ssh service
       }
     ]
 
-## Polling the Journal File
+The `journal_matches` property lets you to filter your journal entries. This property is an optional `[...]` list of "match strings", with the pattern "FIELD=value". FIELD is a field in the journal entry, for example `_SYSTEMD_UNIT`, and "value" is the value to filter that field on. In the above example, only entries from `ssh.service` are imported.
 
-The journald monitor polls the journal file every ``journal_poll_interval`` seconds to check for new logs.  It does this by
-creating a polling object (https://docs.python.org/2/library/select.html#poll-objects) and calling the ``poll`` method
-of that object.  The ``poll`` method is called with a 0 second timeout so it never blocks.
-After processing any new events, or if there are no events to process, the monitor thread sleeps for ``journal_poll_interval``
-seconds and then polls again.
+See [Configuration Options](#options) below for more properties you can add to the `monitors: [ ... ]` section.
+
+
+4\\. Set parsers
+
+In the `agent.json` configuration file, add a top-level `journald_logs: [...]` array. [Modular Configuration](#modular) is supported - you can add this array to any `.json` file in the `agent.d` directory.
+
+    journald_logs: [
+      {
+          journald_unit: ".*",
+          parser: "journaldParser"
+      }
+    ]
+
+`journald_unit` is a regular expression, matching on the journal entry's `_SYSTEMD_UNIT` field. It defaults to `.*`, matching all. Note that double-escaping regex elements is required - see [Regex](https://app.scalyr.com/help/regex) for more information.
+
+The `parser` property creates a [configuration file](https://app.scalyr.com/parsers) in the UI. There you can set rules to parse fields from your journal entries, or you can click a button and we will create a parser for you.
+
+You can set multiple parsers by adding a `{...}` stanza for each. For example, to assign `sshParser` to the logs from the SSH service, and `journaldParser` to the others:
+
+    journald_logs: [
+      {
+           journald_unit: "ssh\\.service",
+           parser: "sshParser",
+           rename_logfile: "/journald/ssh_service.log"
+      },
+      {
+          journald_unit: ".*",
+          parser: "journaldParser"
+      }
+    ]
+
+The parser assigned is based on the first matching entry in the `journald_logs` array.  If you put the first stanza in the above example last, the SSH service will be assigned the "journaldParser", as it matches on `".*"`.
+
+The `rename_logfile` property lets you to create a meaningful name for the log file. This file is auto-generated, with the syntax `/var/log/scalyr-agent-2/journald_XXXX.log`, where `XXXX` is a hash to guarantee the name is unique. In the example above, the SSH service is renamed to "/journald/ssh_service.log".
+
+If you are using Docker and the `journald` log driver, `_SYSTEMD_UNIT` will always be `docker.service`. You can replace the `journald_unit` property with `journald_globs`. This property is a dict of globs, keyed by journald fields. All fields and values defined in `journald_globs` must be present for a match.
+
+Keyed journald fields must also be added to the [journal_fields](#options) property in step **3** above. Only fields defined in this property are available for glob matching.
+
+For example, to assign `sshParser` to the logs from the SSH service, you can use the `_COMM` field:
+
+    monitors: [
+      {
+        module: "scalyr_agent.builtin_monitors.journald_monitor",
+        journal_path: "/run/log/journal",
+        journal_fields: { "_COMM": "process", "_PID": "pid", "_MACHINE_ID": "machine_id", "_BOOT_ID": "boot_id", "_SOURCE_REALTIME_TIMESTAMP": timestamp" }
+      }
+    ]
+
+
+    journald_logs: [
+      {
+          journald_globs: { "_COMM": "ssh" },
+          parser: "sshParser"
+      }
+    ]
+
+
+5\\. Set attributes, and other properties
+
+The `attributes` property lets you attach key-value pairs to your entries. These attributes become searchable fields in the UI, and are especially useful for determining the source of log data.
+
+You can also add and set these properties:
+
+| Property | Description |
+|---|---|
+| [sampling_rules](https://app.scalyr.com/help/scalyr-agent#filter)        | Import a random sample of logs matching a regular expression. Used to decrease log volume. |
+| [exclude](https://app.scalyr.com/help/scalyr-agent#logExclusion)         | A list of glob patterns.  Any file that matches will not be imported. |
+| [redaction_rules](https://app.scalyr.com/help/scalyr-agent#redaction)    | Rewrite or delete sensitive portions of a log. |
+| [compression_type](https://app.scalyr.com/help/scalyr-agent#compressing) | Turn off or change the default `deflate` compression, applied to all imported logs. |
+
+&nbsp;
+
+Here is an example with attributes, redaction rules, and sampling rules set:
+
+    journald_logs: [
+      {
+           journald_unit: "ssh\\.service",
+           parser: "sshParser",
+           attributes: { "service": "ssh" },
+           redaction_rules: [ { match_expression: "password", replacement: "REDACTED" } ],
+           sampling_rules: [ { match_expression: "INFO", sampling_rate: 0.1} ],
+      },
+      {
+          journald_unit: ".*",
+          parser: "journaldParser",
+          attributes: { "service": "unknown" }
+      }
+    ]
+
+
+6\\. Override value escaping (optional)
+
+By default, several quoting and escaping rules are applied. In particular, we quote extracted fields and escape values. You may wish to override this behavior, depending on your journald entry format.
+
+To override quoting and escaping of the journald `MESSAGE` field, add the `emit_raw_details` property, and set it to `true`. This will make parsing easier for messages that are already quoted, and special characters that are already escaped:
+
+    journald_logs: [
+      {
+          journald_unit: ".*",
+          emit_raw_details: true
+      }
+    ]
+
+If `detect_escaped_strings` is `true`, all extracted fields, including `MESSAGE` and fields listed in `journal_fields`, will not be quoted and escaped *if* the first and last character of the field is a quote (`"`). This is useful if you expect any field to be quoted, and special characters escaped:
+
+    journald_logs: [
+      {
+          journald_unit: ".*",
+          detect_escaped_strings: true
+      }
+    ]
+
+
+
+7\\. Save and confirm
+
+Save the `agent.json` file. The Agent will detect changes within 30 seconds. Wait a few minutes for the Scalyr Agent to begin sending data.
+
+You can check the [Agent Status](https://app.scalyr.com/help/scalyr-agent#agentStatus), which includes information about all running monitors.
+
+Log in to your Scalyr account and go to the [Logs](https://app.scalyr.com/logStart) overview page. A log file should show for each `journal_path` you set in step **3**.
+
+For help, contact us at [support@scalyr.com](mailto:support@scalyr.com).
+
+
+<a name="modular"></a>
+## Modular Configuration
+
+You may split your `journald_logs` JSON array across multiple files, similar to how the `logs` JSON array can be split across multiple [modular configuration files](https://app.scalyr.com/help/scalyr-agent#modularConfig), in the `agent.d` directory.
+
+For example, you may have a file `journald_ssh.json` in `agent.d` with contents:
+
+    journald_logs: [
+      {
+           journald_unit: "ssh\\.service",
+           parser: "sshParser",
+           attributes: { "service": "ssh" },
+           redaction_rules: [ { match_expression: "password", replacement: "REDACTED" } ],
+           sampling_rules: [ { match_expression: "INFO", sampling_rate: 0.1} ],
+      }
+    ]
+
+And a second file `zz_journald_defaults.json` with contents:
+
+    journald_logs: [
+      {
+          journald_unit: ".*",
+          parser: "journaldParser",
+          attributes: { "service": "unknown" }
+      }
+    ]
+
+The contents of the `journald_logs` JSON array will be computed by appending all `journald_logs` entries in all files in alphabetical order of their file names.  You can prefix a filename with `zz` to guarantee it will be the last entry in the `journald_logs` JSON array.
     """
     # fmt: on
 
