@@ -39,14 +39,89 @@ MORE_BYTES = re.compile(b"\.\.\. \(\d+ more bytes\)$")  # NOQA
 
 __monitor__ = __name__
 
+define_config_option(
+    __monitor__,
+    "module",
+    "Always `scalyr_agent.builtin_monitors.redis_monitor`",
+    required_option=True,
+)
+
+define_config_option(
+    __monitor__,
+    "hosts",
+    'Optional (defaults to `[{ "host": "localhost", "port": 6379, "password": none}]`). '
+    "An array of host objects; each sets `host`, "
+    "`port`, and `password` properties.",
+    default={"host": "localhost", "port": 6379, "password": None},
+)
+
 define_config_option(
     __monitor__,
     "log_cluster_replication_info",
-    "Optional (defaults to false). If true, the monitor will record redis cluster's replication "
-    "offsets, difference between master and replica, and how many seconds the replica is falling behind",
+    "Optional (defaults to `false`). If `true`, this plugin collects the Redis cluster's "
+    "replication offsets, the difference between master and replica, and how many seconds "
+    "the replica is falling behind.",
     default=False,
 )
 
+define_config_option(
+    __monitor__,
+    "lines_to_fetch",
+    "Optional (defaults to `500`). Number of lines to fetch from the SLOWLOG each `sample_interval`. "
+    "Set this value based on your expected load; some lines will be dropped if "
+    "more than this number of messages are logged to the SLOWLOG between sample intervals. "
+    "Make sure the redis-server `slowlog-max-len` configuration option is set to at least "
+    "the same size. See line 1819 in this [redis.conf](https://www.google.com/url?sa=t&rct=j&q=&esrc=s&source=web&cd=&ved=2ahUKEwiTiNXQoPj3AhXYkokEHYw8AhoQFnoECB0QAQ&url=https%3A%2F%2Fdownload.redis.io%2Fredis-stable%2Fredis.conf&usg=AOvVaw3_kFM1q_S7L9omTrS8uL23) example file. Also set the "
+    "`sample_interval` to match your expected traffic (see below).",
+    default=500,
+    convert_to=int,
+    min_value=10,
+    max_value=5000,
+)
+
+define_config_option(
+    __monitor__,
+    "connection_timeout",
+    "Optional (defaults to `5`). Number of seconds to wait when querying a Redis server "
+    "before timing out and giving up.",
+    default=5,
+    convert_to=int,
+    min_value=0,
+    max_value=60,
+)
+
+define_config_option(
+    __monitor__,
+    "connection_error_repeat_interval",
+    "Optional (defaults to `300`). Seconds to wait before repeating an "
+    "error message (when multiple errors are detected).",
+    default=300,
+    convert_to=int,
+    min_value=10,
+    max_value=6000,
+)
+
+define_config_option(
+    __monitor__,
+    "utf8_warning_interval",
+    "Optional (defaults to `600`). Minimum seconds to wait between "
+    "warnings about invalid utf8 in redis slow log messages. Set to 0 to disable "
+    "the warning altogether.",
+    default=600,
+    convert_to=int,
+    min_value=0,
+    max_value=6000,
+)
+
+define_config_option(
+    __monitor__,
+    "sample_interval",
+    "Optional (defaults to `30`). Seconds to wait between queries "
+    "to the Redis SLOWLOG. If you expect a high volume of logs, set this value low "
+    "enough so the number of messages you receive in this interval does not "
+    "exceed the value of `lines_to_fetch`, otherwise some lines will be dropped.",
+)
+
 
 class RedisHost(object):
     """Class that holds various information about a specific redis connection"""
@@ -342,22 +417,27 @@ class RedisHost(object):
 
 class RedisMonitor(ScalyrMonitor):  # pylint: disable=monitor-not-included-for-win32
     # fmt: off
-    """
-# Redis Monitor
+    r"""
+# Redis
 
-A Scalyr agent monitor that imports the Redis SLOWLOG.
+Import the Redis SLOWLOG.
 
-The Redis monitor queries the slowlog of a number of Redis servers, and uploads the logs
-to the Scalyr servers.
+An [Agent Plugin](https://app.scalyr.com/help/scalyr-agent#plugins) is a component of the Scalyr Agent, enabling the collection of more data. The source code for each plugin is available on [Github](https://github.com/scalyr/scalyr-agent-2/tree/master/scalyr_agent/builtin_monitors).
 
-@class=bg-warning docInfoPanel: An *agent monitor plugin* is a component of the Scalyr Agent. To use a plugin,
-simply add it to the ``monitors`` section of the Scalyr Agent configuration file (``/etc/scalyr/agent.json``).
-For more information, see [Agent Plugins](/help/scalyr-agent#plugins).
 
-## Sample Configuration
+## Installation
 
-The following example will configure the agent to query a redis server located at ``localhost:6379`` and that does not require
-a password
+
+1\. Install the Scalyr Agent
+
+If you haven't already, install the [Scalyr Agent](https://app.scalyr.com/help/welcome). We recommend you install the Agent on each server running Redis. Your data will automatically be tagged for the server it came from, and the Agent can also collect system metrics and log files.
+
+
+2\. Configure the Scalyr Agent to import the Redis SLOWLOG
+
+Open the Scalyr Agent configuration file, located at `/etc/scalyr-agent-2/agent.json`.
+
+Find the `monitors: [ ... ]` section and add a `{...}` stanza with the `module` property set for Redis:
 
     monitors: [
       {
@@ -365,7 +445,7 @@ a password
       }
     ]
 
-Here is an example with two hosts with passwords:
+This default configuration is for a Redis server without a password, located at `localhost:6379`. To change these defaults, add a `hosts [...]` array. Then add and set a `{...}` stanza for each host with the applicable `host`, `port`, and `password` properties. An example for two hosts, with passwords:
 
     monitors: [
       {
@@ -377,44 +457,18 @@ Here is an example with two hosts with passwords:
       }
     ]
 
-*   hosts - an array of 'host' objects. Each host object can contain any or all of the following keys: "host", "port", "password".
-    Missing keys will be filled in with the defaults: 'localhost', 6379, and <None>.
-
-    For example:
-
-    hosts: [
-        { "host": "redis.example.com", "password": "secret" },
-        { "port": 6380 }
-    ]
-
-    Will create connections to 2 Redis servers: redis.example.com:6379 with the password 'secret' and
-    localhost:6380 without a password.
-
-    Each host in the list will be queried once per sample interval.
-
-*   lines_to_fetch - the number of lines to fetch from the slowlog each sample interval.  Defaults to 500.
-    This value should be set based on your expected load.  If more than this number of messages are logged to the
-    slowlog between sample intervals then some lines will be dropped.
-
-    You should make sure that the redis-server slowlog-max-len config option is set to at least the same size as
-    this value.
-
-    You should also set your sample_interval to match your expected traffic (see below).
-
-*   connection_timeout - the number of seconds to wait when querying a Redis server before timing out and giving up.
-    Defaults to 5.
-
-*   connection_error_repeat_interval - if multiple connection errors are detected, the number of seconds to wait before
-    redisplaying an error message.  Defaults to 300.
-
-*   utf8_warning_interval - the minimum amount of time in seconds to wait between issuing warnings about invalid utf8 in redis slow
-    log messages.  Set to 0 to disable the warning altogether. Defaults to 600
+See [Configuration Options](#options) below for more properties you can add.
 
 
-*   sample_interval - the number of seconds to wait between successive queryies to the Redis slowlog.  This defaults to 30 seconds
-    for most monitors, however if you are expecting a high volume of logs you should set this to a low enough value
-    such that the number of messages you receive in this interval does not exceed the value of `lines_to_fetch`, otherwise
-    some log lines will be dropped.
+3\. Save and confirm
+
+Save the `agent.json` file. The Agent will detect changes within 30 seconds. Wait a few minutes for the Agent to begin sending the Redis SLOWLOG.
+
+You can check the [Agent Status](https://app.scalyr.com/help/scalyr-agent#agentStatus), which includes information about all running monitors.
+
+Log into Scalyr. From Search view query [monitor = 'redis_monitor'](/events&filter=monitor+%3D+%27redis_monitor%27).
+
+For help, contact Support.
 
     """
     # fmt: on
