@@ -12,12 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ------------------------------------------------------------------------
-# author:  Imron Alston <imron@scalyr.com>
+# author: scalyr-cloudtech@scalyr.com
 
 from __future__ import unicode_literals
 from __future__ import absolute_import
 
+import copy
+import collections
 import os
+import re
 import threading
 import time
 
@@ -41,10 +44,10 @@ except ImportError as e:
     windll = None
 
 import scalyr_agent.util as scalyr_util
-from scalyr_agent import ScalyrMonitor, define_config_option
+from scalyr_agent import ScalyrMonitor, define_config_option, define_log_field
 import scalyr_agent.scalyr_logging as scalyr_logging
 
-__author__ = "imron@scalyr.com"
+__author__ = "scalyr-cloudtech@scalyr.com"
 
 __monitor__ = __name__
 
@@ -56,7 +59,7 @@ DEFAULT_MAXIMUM_RECORDS_PER_SOURCE = 10000
 define_config_option(
     __monitor__,
     "module",
-    "Always ``scalyr_agent.builtin_monitors.windows_event_log_monitor``",
+    "Always `scalyr_agent.builtin_monitors.windows_event_log_monitor`",
     convert_to=six.text_type,
     required_option=True,
 )
@@ -64,9 +67,9 @@ define_config_option(
 define_config_option(
     __monitor__,
     "sources",
-    "Optional (defaults to ``Application, Security, System``). A comma separated list of event sources."
-    "You can use this to specify which event sources you are interested in listening to."
-    '(Vista and later) Cannot be used.  Please use the "channels" parameter instead.',
+    "Optional (defaults to `Application, Security, System`). A comma separated list "
+    "of sources to import events from. (Not valid for Vista and later; use `channels` "
+    "instead.)",
     convert_to=six.text_type,
     default=DEFAULT_SOURCES,
 )
@@ -74,9 +77,9 @@ define_config_option(
 define_config_option(
     __monitor__,
     "event_types",
-    "Optional (defaults to ``All``). A comma separated list of event types to log."
-    "Valid values are: All, Error, Warning, Information, AuditSuccess and AuditFailure"
-    '(Vista and later) Cannot be used.  Please use the "channels" parameter instead.',
+    "Optional (defaults to `All`). A comma separated list of event types to import. "
+    "Valid values are: `All`, `Error`, `Warning`, `Information`, `AuditSuccess`, and "
+    "`AuditFailure`. (Not valid for Vista and later; use `channels` instead.)",
     default=DEFAULT_EVENTS,
     convert_to=six.text_type,
 )
@@ -84,25 +87,24 @@ define_config_option(
 define_config_option(
     __monitor__,
     "channels",
-    "A list of dict objects specifying a list of channels and an XPath query for those channels."
-    "Only available on Windows Vista and later."
-    'Optional (defaults to ``[ {"channel" : ["Application", "Security", "System"], "query": "*"}]',
+    'Optional (defaults to `[{"channel" : ["Application", "Security", "System"], "query": "*"}]`). '
+    "A list of dict objects specifying a list of channels, and an XPath query for those channels. "
+    "(Only available on Windows Vista and later.)",
     convert_to=None,
 )
 
 define_config_option(
     __monitor__,
     "maximum_records_per_source",
-    "Optional (defaults to ``10000``). The maximum number of records to read from the end of each log source"
-    "per gather_sample. Only applicable when using old WEL API.",
-    default=DEFAULT_MAXIMUM_RECORDS_PER_SOURCE,
+    "Optional (defaults to `10000`). Maximum number of records to read from the end of each "
+    "log source per gather_sample.",
     convert_to=int,
 )
 
 define_config_option(
     __monitor__,
     "error_repeat_interval",
-    "Optional (defaults to ``300``). The number of seconds to wait before logging similar errors in the event log.",
+    "Optional (defaults to `300`). Number of seconds to wait before logging similar errors in the event log.",
     default="300",
     convert_to=int,
 )
@@ -110,7 +112,7 @@ define_config_option(
 define_config_option(
     __monitor__,
     "server_name",
-    "Optional (defaults to ``localhost``). The remote server where the event log is to be opened.",
+    "Optional (defaults to `localhost`). The remote server to import events from.",
     default="localhost",
     convert_to=six.text_type,
 )
@@ -118,7 +120,8 @@ define_config_option(
 define_config_option(
     __monitor__,
     "remote_user",
-    "Optional (defaults to ``None``). The username to use for authentication on the remote server.  This option is only valid on Windows Vista and above.",
+    "Optional (defaults to `none`). Username for authentication on the remote server. "
+    "This option is only valid on Windows Vista and above.",
     default=None,
     convert_to=six.text_type,
 )
@@ -126,7 +129,8 @@ define_config_option(
 define_config_option(
     __monitor__,
     "remote_password",
-    "Optional (defaults to ``None``). The password to use for authentication on the remote server.  This option is only valid on Windows Vista and above.",
+    "Optional (defaults to `none`). Password to use for authentication on the remote server.  "
+    "This option is only valid on Windows Vista and above.",
     default=None,
     convert_to=six.text_type,
 )
@@ -134,7 +138,8 @@ define_config_option(
 define_config_option(
     __monitor__,
     "remote_domain",
-    "Optional (defaults to ``None``). The domain to which the remote user account belongs.  This option is only valid on Windows Vista and above.",
+    "Optional (defaults to `none`). The domain for the remote user account. "
+    "This option is only valid on Windows Vista and above.",
     default=None,
     convert_to=six.text_type,
 )
@@ -142,11 +147,125 @@ define_config_option(
 define_config_option(
     __monitor__,
     "json",
-    "Optional (defaults to ``False``). Format events as json? Supports inclusion of all event fields. This option is only valid on Windows Vista and above.",
+    "Optional (defaults to `false`). Format events as json? Supports inclusion of "
+    "all event fields. This option is only valid on Windows Vista and above.",
     default=False,
     convert_to=bool,
 )
 
+define_config_option(
+    __monitor__,
+    "placeholder_render",
+    "Optional (defaults to `false`). Render %%n placeholders in event data? This option is only "
+    "valid on Windows Vista and above.",
+    default=False,
+    convert_to=bool,
+)
+
+define_config_option(
+    __monitor__,
+    "dll_handle_cache_size",
+    "Optional (defaults to `10`). DLL handle cache size, applicable only if `placeholder_render` "
+    "is set. This option is only valid on Windows Vista and above.",
+    default=10,
+    convert_to=int,
+)
+
+define_config_option(
+    __monitor__,
+    "placeholder_param_cache_size",
+    "Optional (defaults to `1000`). Placeholder parameter cache size, applicable only if "
+    "`placeholder_render` is set. This option is only valid on Windows Vista and above.",
+    default=1000,
+    convert_to=int,
+)
+
+define_config_option(
+    __monitor__,
+    "dll_handle_cache_ttl",
+    "Optional (defaults to `86400` ie 24 hours). DLL handle cache TTL, applicable only if "
+    "`placeholder_render` is set. This option is only valid on Windows Vista and above.",
+    default=86400,
+    convert_to=int,
+)
+
+define_config_option(
+    __monitor__,
+    "placeholder_param_cache_ttl",
+    "Optional (defaults to `86400` ie 24 hours). Placeholder parameter cache TLL, applicable only "
+    "if `placeholder_render` is set. This option is only valid on Windows Vista and above.",
+    default=86400,
+    convert_to=int,
+)
+
+define_log_field(__monitor__, "monitor", "Always ``windows_event_log_monitor``.")
+
+
+define_log_field(
+    __monitor__,
+    "monitor",
+    "Always `windows_event_log_monitor`.",
+)
+define_log_field(
+    __monitor__,
+    "Channel",
+    "The event channel name, taken from the `Event.System.Channel` field. "
+    "Only for Vista and above; see `Source` for pre-Vista versions.",
+)
+define_log_field(
+    __monitor__,
+    "EventRecordID",
+    "The event record number, taken from the `Event.System.EventRecordID` field. "
+    "Only for Vista and above; see `RecordNumber` for pre-Vista versions.",
+)
+define_log_field(
+    __monitor__,
+    "SystemTime",
+    "The time the event was generated, taken from the `Event.System.TimeCreated.SystemTime` "
+    "field. Only for Vista and above; see `RecordNumber` for pre-Vista versions.",
+)
+define_log_field(
+    __monitor__,
+    "EventId",
+    "The event id, taken from the `Event.System.EventID` field on Vista and above, "
+    "and from `event.EventID` for pre-Vista versions.",
+)
+define_log_field(
+    __monitor__,
+    "Source",
+    "The event source name, taken from the `event.SourceName` field. "
+    "Only for pre-Vista versions of Windows; see `Channel` for Vista and above.",
+)
+define_log_field(
+    __monitor__,
+    "RecordNumber",
+    "The event record number, taken from the `event.RecordNumber` field. "
+    "Only for pre-Vista versions of Windows; see `SystemTime` for Vista and above.",
+)
+define_log_field(
+    __monitor__,
+    "TimeGenerated",
+    "The time the event was generated. "
+    "Only for pre-Vista versions of Windows; see `SystemTime` for Vista and above.",
+)
+define_log_field(
+    __monitor__,
+    "Type",
+    "The event type. Only for pre-Vista versions of Windows.",
+)
+define_log_field(
+    __monitor__,
+    "Category",
+    "The event category, taken from the `event.EventCategory` field. "
+    "Only for pre-Vista versions of Windows.",
+)
+define_log_field(
+    __monitor__,
+    "EventMsg",
+    "The contents of the event message from the Windows Event Log. "
+    "Only for pre-Vista versions of Windows.",
+)
+
 
 class Api(object):
     def __init__(self, config, logger):
@@ -653,14 +772,39 @@ class NewJsonApi(NewApi):
             win32evtlog.EvtRenderContextSystem
         )
 
+        self._placeholder_render = config.get("placeholder_render")
+        if self._placeholder_render:
+            self._dll_cache = Cache(
+                size=config.get("dll_handle_cache_size"),
+                ttl=config.get("dll_handle_cache_ttl"),
+            )
+            self._param_cache = Cache(
+                size=config.get("placeholder_param_cache_size"),
+                ttl=config.get("placeholder_param_cache_ttl"),
+            )
+
+            self._cache_report_interval = 3600 * 12
+            self._next_cache_report = time.time() + self._cache_report_interval
+
+            self._langid = win32api.MAKELANGID(
+                win32con.LANG_NEUTRAL, win32con.SUBLANG_NEUTRAL
+            )
+
     def log_event(self, event):
         values = win32evtlog.EvtRender(
             event, win32evtlog.EvtRenderEventValues, Context=self._render_context
         )
 
-        metadata = win32evtlog.EvtOpenPublisherMetadata(
-            values[win32evtlog.EvtSystemProviderName][0]
-        )
+        # (Publisher) metadata is used to populate the RenderingInfo section,
+        # which is already populated for forwarded events and would throw an exception here.
+        # Ref: https://docs.microsoft.com/en-us/windows/win32/api/winevt/nf-winevt-evtformatmessage
+        metadata = None
+        try:
+            metadata = win32evtlog.EvtOpenPublisherMetadata(
+                values[win32evtlog.EvtSystemProviderName][0]
+            )
+        except Exception:
+            pass
 
         event_json = xmltodict.parse(
             win32evtlog.EvtFormatMessage(
@@ -676,6 +820,20 @@ class NewJsonApi(NewApi):
         #        => '{"entry": {"text": "bar", "#text": "foo"}}'
         event_json = _strip_xmltodict_prefixes(event_json)
 
+        # FormatMessage will replace insertion and parameter strings in the event message,
+        # however it will not replace parameter strings in the event data.
+        # Ref: https://learn.microsoft.com/en-us/windows/win32/eventlog/event-identifiers#message-definitions
+        #      https://learn.microsoft.com/en-us/windows/win32/eventlog/message-files
+        #      FormatMessage of https://github.com/mhammond/pywin32/blob/main/win32/Lib/win32evtlogutil.py
+        if self._placeholder_render:
+            event_json = self._replace_param_placeholders(event_json)
+            if time.time() > self._next_cache_report:
+                self._next_cache_report = time.time() + self._cache_report_interval
+                self._logger.info(
+                    "placeholder param cache size = %d" % len(self._param_cache)
+                )
+                self._logger.info("dll handle cache size = %d" % len(self._dll_cache))
+
         # Populate the record here with fields that would normally be added by the log formatter,
         # this avoids having to unmarshal and remarshal later in the log formatter.
         # Refer to the use of DummyFormatter in WindowEventLogMonitor.open_metric_log().
@@ -683,6 +841,7 @@ class NewJsonApi(NewApi):
             "SystemTime"
         ]
         event_json["name"] = self._logger.name
+        event_json["monitor"] = "windows_event_log_monitor"
         self._logger.emit_value("unused", scalyr_util.json_encode(event_json))
 
         self._bookmark_lock.acquire()
@@ -694,10 +853,125 @@ class NewJsonApi(NewApi):
         finally:
             self._bookmark_lock.release()
 
+    def _param_placeholder_value(self, channel, provider, param):
+        if not re.match("^%%[0-9]+$", param):
+            return param
+
+        if channel == "Security":
+            provider = "Security"
+
+        param_key = "%s-%s-%s" % (channel, provider, param)
+        if param_key in self._param_cache:
+            return self._param_cache[param_key]
+
+        dll_key = "%s-%s" % (channel, provider)
+        if dll_key in self._dll_cache:
+            if not isinstance(self._dll_cache[dll_key], _DLL):
+                return param
+        else:
+            # Before opening a new handle to a dll check if another handle to the same dll is open;
+            # multiple event log providers may ultimately reference the same dll.
+
+            try:
+                dllpath = _DLL.dllpath(channel, provider)
+            except Exception as e:
+                self._dll_cache[dll_key] = e
+                return param
+
+            found = False
+            for dll in self._dll_cache.values():
+                if not isinstance(dll, _DLL):
+                    continue
+                if dll.path == dllpath:
+                    self._dll_cache[dll_key] = dll
+                    found = True
+                    break
+
+            if not found:
+                try:
+                    self._dll_cache[dll_key] = _DLL(channel, provider)
+                except Exception as e:
+                    self._dll_cache[dll_key] = e
+                    self._logger.error(
+                        "win32api.LoadLibraryEx exception: %s" % six.text_type(e),
+                        limit_once_per_x_secs=self._error_repeat_interval,
+                        limit_key="win32api.LoadLibraryEx",
+                    )
+                    return param
+
+        try:
+            value = win32api.FormatMessageW(
+                win32con.FORMAT_MESSAGE_FROM_HMODULE,
+                self._dll_cache[dll_key].handle,
+                int(param[2:]),
+                self._langid,
+                None,
+            )
+            self._param_cache[param_key] = value.strip()
+        except Exception as e:
+            self._param_cache[param_key] = param
+            self._logger.error(
+                "win32api.FormatMessageW exception: %s" % six.text_type(e),
+                limit_once_per_x_secs=self._error_repeat_interval,
+                limit_key="win32api.FormatMessageW",
+            )
+
+        return self._param_cache[param_key]
+
+    def _replace_param_placeholders(self, event):
+        rv = copy.deepcopy(event)
+
+        try:
+            event_data = event["Event"]["EventData"]["Data"]
+            channel = event["Event"]["System"]["Channel"]
+            provider = event["Event"]["System"]["Provider"]["Name"]
+        except:
+            return rv
+
+        if isinstance(event_data, str) and re.match("^%%[0-9]+$", event_data):
+            rv["Event"]["EventData"]["Data"] = self._param_placeholder_value(
+                channel, provider, event_data
+            )
+
+        elif isinstance(event_data, dict):
+            for key, val in event_data.items():
+                if isinstance(val, str) and re.match("^%%[0-9]+$", val):
+                    rv["Event"]["EventData"]["Data"][
+                        key
+                    ] = self._param_placeholder_value(channel, provider, val)
+                elif (
+                    isinstance(val, dict)
+                    and "Text" in val
+                    and re.match("^%%[0-9]+$", val["Text"])
+                ):
+                    rv["Event"]["EventData"]["Data"][key][
+                        "Text"
+                    ] = self._param_placeholder_value(channel, provider, val["Text"])
+
+        return rv
+
 
 def _convert_json_array_to_object(x):
     if isinstance(x, list):
-        return {str(i): _convert_json_array_to_object(x[i]) for i in range(len(x))}
+        # Special handling of @Name fields per customer request:
+        # Ie [{"@Name:":"n1", "a1":"v1", ...}, ...] becomes {"n1":{"a1":"v1", ...}, ...}
+        # Duplicate @Names are handled by appending the index number if possible.
+        # Otherwise the (stringified) index number becomes the field name.
+        rv = {}
+        for i in range(len(x)):
+            if isinstance(x[i], dict) and "@Name" in x[i]:
+                orig_name = name = x[i].pop("@Name")
+                if name in rv:
+                    name += str(i)
+                    # Fallback to the original behavior if this pathological case occurs
+                    if name in rv:
+                        x[i]["@Name"] = orig_name
+                        rv[str(i)] = _convert_json_array_to_object(x[i])
+                        continue
+                rv[name] = _convert_json_array_to_object(x[i])
+            else:
+                rv[str(i)] = _convert_json_array_to_object(x[i])
+        return rv
     elif isinstance(x, dict):
         return {k: _convert_json_array_to_object(v) for k, v in x.items()}
     else:
@@ -726,66 +1000,204 @@ def _strip_xmltodict_prefixes(x):
         return x
 
 
+class _DLL:
+    @staticmethod
+    def dllpath(channel, provider):
+        keyname = "SYSTEM\\CurrentControlSet\\Services\\EventLog\\%s\\%s" % (
+            channel,
+            provider,
+        )
+        keyhandle = win32api.RegOpenKey(win32con.HKEY_LOCAL_MACHINE, keyname)
+        try:
+            return win32api.ExpandEnvironmentStrings(
+                win32api.RegQueryValueEx(keyhandle, "ParameterMessageFile")[0]
+            )
+        finally:
+            win32api.RegCloseKey(keyhandle)
+
+    def __init__(self, channel, provider):
+        try:
+            self.path = self.dllpath(channel, provider)
+            self.handle = win32api.LoadLibraryEx(
+                self.path, 0, win32con.LOAD_LIBRARY_AS_DATAFILE
+            )
+        except Exception as e:
+            self.handle = None
+            raise e
+
+    def __del__(self):
+        if self.handle:
+            win32api.FreeLibrary(self.handle)
+            self.handle = None
+
+
+class Cache(collections.OrderedDict):
+    """
+    Fixed-size cache with entry TTL
+    """
+
+    def __init__(self, size, ttl):
+        super().__init__()
+        self.__size = size
+        self.__ttl = ttl
+
+    def __setitem__(self, key, val):
+        while len(self) >= self.__size:
+            self.popitem(last=False)
+        super().__setitem__(key, (val, time.time()))
+
+    def __getitem__(self, key):
+        # Intentionally not evicting entries due to ttl here.
+        # Expecting checks via __contains__ followed immediately by __getitem__
+        val, _ = super().__getitem__(key)
+        return val
+
+    def __contains__(self, key):
+        try:
+            _, added = super().__getitem__(key)
+        except KeyError:
+            return False
+
+        if time.time() > added + self.__ttl:
+            del self[key]
+            return False
+        return True
+
+    def __iter__(self):
+        active = []
+        expired = []
+        for key in super().__iter__():
+            _, added = super().__getitem__(key)
+
+            if time.time() > added + self.__ttl:
+                expired += [key]
+            else:
+                active += [key]
+
+        for key in expired:
+            del self[key]
+        return iter(active)
+
+    def values(self):
+        active_vals = []
+        expired_keys = []
+        for key in super().__iter__():
+            val, added = super().__getitem__(key)
+
+            if time.time() > added + self.__ttl:
+                expired_keys += [key]
+            else:
+                active_vals += [val]
+
+        for key in expired_keys:
+            del self[key]
+        return iter(active_vals)
+
+    def __len__(self):
+        return len(list(iter(self)))
+
+
 class WindowEventLogMonitor(ScalyrMonitor):
     # fmt: off
-    """
-# Window Event Log Monitor
+    r"""
+# Windows Event Log
 
-The Windows Event Log monitor uploads messages from the Windows Event Log to the Scalyr servers.
-It can listen to multiple different event sources and also filter by messages of a certain type.
+Import messages from the Windows Event Log.
 
-On versions of Windows prior to Vista, the older EventLog API is used.  This API is unable to
-retrieve 'Critical' events because this event type was only introduced in Vista.
+An [Agent Plugin](https://app.scalyr.com/help/scalyr-agent#plugins) is a component of the Scalyr Agent, enabling the collection of more data. The source code for each plugin is available on [Github](https://github.com/scalyr/scalyr-agent-2/tree/master/scalyr_agent/builtin_monitors).
 
-On versions of Windows from Vista onwards, the newer Evt API is used which can be used to retrieve
-'Critical' events.
+You can import events from multiple channels or sources, for example Application, Security, and System events. You can also filter by event type, for example "System/Level=0" events.
 
-@class=bg-warning docInfoPanel: An *agent monitor plugin* is a component of the Scalyr Agent. To use a plugin,
-simply add it to the ``monitors`` section of the Scalyr Agent configuration file (``/etc/scalyr/agent.json``).
-For more information, see [Agent Plugins](/help/scalyr-agent#plugins).
+On Windows versions before Vista, 'Critical' events do not exist, and the older EventLog API is used. From Vista onwards, the newer Evt API is used.
 
-## Sample Configuration
 
-### Windows Vista and later
+## Installation
 
-On Windows Vista and later, the Scalyr agent uses the EvtLog API, and you can configure it to query events on any channel, using the standard XPath query mechanism.  See: https://msdn.microsoft.com/en-us/library/windows/desktop/dd996910(v=vs.85).aspx
+1\. Install the Scalyr Agent
 
-For example, the following will configure the agent to listen to Critical, Error and Warning level events from the Application, Security and System channels:
+If you haven't already, install the [Scalyr Agent](https://app.scalyr.com/help/welcome) on the Windows server.
+
+
+2\. Configure the Scalyr Agent to import Event Logs
+
+Open the `agent.json` configuration file, located at `C:\\Program Files (x86)\\Scalyr\\Config`.
+
+Find the `monitors: [ ... ]` section and add a `{...}` stanza with the `module` property set for the windows event log:
 
     monitors: [
       {
-        module:                  "scalyr_agent.builtin_monitors.windows_event_log_monitor",
+         module:      "scalyr_agent.builtin_monitors.windows_event_log_monitor",
+      }
+    ]
+
+
+### Windows Vista and Later
+
+You can configure the EvtLog API to query events on any channel with XPath. See the
+[Event log](https://msdn.microsoft.com/en-us/library/windows/desktop/dd996910.aspx) documentation for more
+details. For example:
+
+    monitors: [
+      {
+        module: "scalyr_agent.builtin_monitors.windows_event_log_monitor",
         channels: [
             { "channel": [ "Application", "Security", "System" ],
-              "query": "*[System/Level=1 or System/Level=2 or System/Level=3]"
+              "query": "*[System/Level=0 or System/Level=1 or System/Level=2 or System/Level=3 or System/Level=4]"
             }
         ]
       }
     ]
 
-Alternatively, here is a configuration that will log critical errors for the Application channel, and critical, error and warning messages for System and Security channels.
+`channels` is a list of `{...}` dict objects, each with `channel` and `query` properties. When not set it defaults to `[ {"channel" : ["Application", "Security", "System"], "query": "*"}]`, which imports all events from the Application, Security, and System channels.
+
+The `channel` property is a list of channels to import events from, and the `query` property is an XPath expression. Events matching the `query`, from the channels in `channel`, are imported. In the above example, the Agent will import Critical (1), Error (2), Warning (3), and Information (4) events from the Application, Security and System channels.
+
+This example imports Critical (1) events for the Application channel; and Critical (1), Error (2), and Warning (3) events for the System and Security channels:
 
     monitors: [
       {
-        module:                  "scalyr_agent.builtin_monitors.windows_event_log_monitor",
+        module: "scalyr_agent.builtin_monitors.windows_event_log_monitor",
         channels: [
             { "channel": ["Application"],
               "query": "*[System/Level=1]"
             },
             {
               "channel": ["Security", "System" ],
-              "query": "*[System/Level=1 or System/Level=2 or System/Level=3]"
+              "query": "*[System/Level=0 or System/Level=1 or System/Level=2 or System/Level=3]"
             }
         ]
       }
     ]
 
+
+You can also select events with the `<Channel>` tag, in the XML of an event. Go to:
+
+    Run > eventvwr.msc > *select event you want to import* > Event Properties > Details > Select XML
+
+For example, Microsoft-Windows-AAD/Operational events have the tag:
+
+    <Channel>Microsoft-Windows-AAD/Operational</Channel>
+
+To configure the Agent to listen to Critical (1), Error (2), Warning (3), and Information (4) events from this channel:
+
+    monitors: [
+      {
+        module: "scalyr_agent.builtin_monitors.windows_event_log_monitor",
+        channels: [
+            { "channel": ["Microsoft-Windows-AAD/Operational"],
+              "query": "*[System/Level=0 or System/Level=1 or System/Level=2 or System/Level=3 or System/Level=4]"
+            }
+        ]
+      }
+    ]
+
+
 ### Windows Server 2003
 
-For Windows versions earlier than Vista, the Scalyr agent will use the older Event Log API.
+For Windows versions earlier than Vista, the Agent uses the older Event Log API.
 
-This sample will configure the agent running on Windows Server 2003 to listen to Error and Warning level events from the Application, Security
-and System sources:
+This example configures the Agent, running on Windows Server 2003, to import Error and Warning level events from
+the Application, Security and System sources:
 
     monitors: [
       {
@@ -794,6 +1206,38 @@ and System sources:
         event_types:             "Error, Warning",
       }
     ]
+
+`sources` is a comma separated list of event sources to import events from. It defaults to `Application, Security, System`. `event_types` is a comma separated list of event types to import. Valid values are: `All` (the default), `Error`, `Warning`, `Information`, `AuditSuccess` and `AuditFailure`. In the above example, only Error and Warning events from Application, Security, and System sources are imported.
+
+
+3\. (Optional) Set more configuration options
+
+You can format imported events as JSON. Simply add and set `json: true` in the configuration file. For example:
+
+    monitors: [
+      {
+        module: "scalyr_agent.builtin_monitors.windows_event_log_monitor",
+        json: true,
+        channels: [
+            { "channel": [ "Application", "Security", "System" ],
+              "query": "*[System/Level=0 or System/Level=1 or System/Level=2 or System/Level=3 or System/Level=4]"
+            }
+        ]
+      }
+    ]
+
+
+See [Configuration Options](#options) below for more options. You can set a remote domain, server, username, and password; the number of records to read from the end of each log source; and the time to wait before logging similar errors.
+
+
+4\. Save and confirm
+
+Save the `agent.json` file. The Agent will detect changes within 30 seconds. Wait a few minutes for data to send.
+
+You can check the [Agent Status](https://app.scalyr.com/help/scalyr-agent#agentStatus), which includes information about all running monitors.
+
+Log into DataSet and query [monitor = 'windows_process_metrics'](https://app.scalyr.com/events?filter=monitor+%3D+%27windows_event_log%27). This will show all data collected by this plugin, across all servers.
+
     """
     # fmt: on
 
@@ -900,8 +1344,13 @@ and System sources:
         return result
 
     def open_metric_log(self):
-        class DummyFormatter:
+        class DummyFormatter(scalyr_logging.MetricLogFormatter):
             def format(self, record):
+                # Ensure the LogRecord has a message attribute.
+                # If the rate limit is disabled, formatting is not applied by RateLimitLogFilter.
+                # Ref: scalyr_logging.MetricLoggingHandler.__init__
+                if not hasattr(record, "message"):
+                    super(DummyFormatter, self).format(record)
                 return record.message[len("unused ") + 1 : -1].replace('\\"', '"')
 
         rv = super(WindowEventLogMonitor, self).open_metric_log()
