@@ -99,32 +99,35 @@ global_log = scalyr_logging.getLogger(__name__)
 CONFIG_OPTIONS = [
     dict(
         option_name="module",
-        option_description="Always ``scalyr_agent.builtin_monitors.windows_process_metrics``",
+        option_description="Always `scalyr_agent.builtin_monitors.windows_process_metrics`",
         convert_to=six.text_type,
         required_option=True,
     ),
+    dict(
+        option_name="id",
+        option_description="An id, included with each event. Shows in the UI as a value "
+        "for the `instance` field. This is especially useful if you are running multiple "
+        "instances of this plugin to import metrics from multiple processes. Each instance "
+        "has a separate `{...}` stanza in the configuration file "
+        r"(`C:\Program Files (x86)\Scalyr\config\agent.json`).",
+        required_option=True,
+        convert_to=six.text_type,
+    ),
     dict(
         option_name="commandline",
-        option_description="A regular expression which will match the command line or name of the process you're "
-        "interested in, as shown in the output of ``tasklist`` or ``wmic process list``. (If "
-        "multiple processes match the same command line pattern, only one will be monitored.)",
+        option_description="A regular expression, matching on the command line output "
+        "of `tasklist`, or `wmic process list`. Selects the process of interest. "
+        "If multiple processes match, only metrics from the first match are imported.",
         default=None,
         convert_to=six.text_type,
     ),
     dict(
         option_name="pid",
-        option_description="The pid of the process from which the monitor instance will collect metrics.  This is "
-        "ignored if the ``commandline`` is specified.",
+        option_description="Process identifier (PID). An alternative to `commandline` "
+        "to select a process. If `commandline` is set, this property is ignored.",
         default=None,
         convert_to=six.text_type,
     ),
-    dict(
-        option_name="id",
-        option_description="Included in each log message generated by this monitor, as a field named ``instance``. "
-        "Allows you to distinguish between values recorded by different instances of this monitor.",
-        required_option=True,
-        convert_to=six.text_type,
-    ),
 ]
 
 _ = [
@@ -217,7 +220,8 @@ _PROCESS_CPU_METRICS = [
     METRIC(  # ------------------ User-mode CPU ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.cpu",
-            description="The number of seconds the CPU has spent executing instructions in user space.",
+            description="Seconds of user space CPU execution. The value is cumulative since process "
+            "start; see `winproc.uptime`.",
             category="CPU",
             unit="secs",
             cumulative=True,
@@ -228,7 +232,8 @@ _PROCESS_CPU_METRICS = [
     METRIC(  # ------------------ Kernel-mode CPU ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.cpu",
-            description="The number of seconds the CPU has spent executing instructions in kernel space.",
+            description="Seconds of kernel space CPU execution. The value is cumulative since "
+            "process start; see `winproc.uptime`.",
             category="CPU",
             unit="secs",
             cumulative=True,
@@ -249,7 +254,7 @@ _PROCESS_ATTRIBUTE_METRICS = [
     METRIC(  # ------------------  Process Uptime   ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.uptime",
-            description="The number of seconds since the process was created.",
+            description="Process uptime, in seconds.",
             category="General",
             unit="seconds",
             cumulative=True,
@@ -260,7 +265,7 @@ _PROCESS_ATTRIBUTE_METRICS = [
     METRIC(  # ------------------  Process Threads   ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.threads",
-            description="The number of threads being used by the process.",
+            description="Number of threads used by the process.",
             category="General",
             extra_fields={},
         ),
@@ -284,8 +289,8 @@ _PROCESS_MEMORY_METRICS = [
     METRIC(  # ------------------ Working Set ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.mem.bytes",
-            description="The number of bytes of physical memory used by the process's working set.  This "
-            "is the amount of memory that needs to be paged in for the process to execute.",
+            description="Bytes of physical memory used by the process's working set. "
+            "Memory that must be paged in for the process to execute.",
             category="Memory",
             unit="bytes",
             extra_fields={"type": "working_set"},
@@ -295,7 +300,7 @@ _PROCESS_MEMORY_METRICS = [
     METRIC(  # ------------------ Peak Working Set ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.mem.bytes",
-            description="The peak working set size for the process since creation time, in bytes.",
+            description="Peak working set size, in bytes, for the process since creation.",
             category="Memory",
             unit="bytes",
             extra_fields={"type": "peak_working_set"},
@@ -305,7 +310,7 @@ _PROCESS_MEMORY_METRICS = [
     METRIC(  # ------------------ Paged Pool ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.mem.bytes",
-            description="The paged-pool usage, in bytes.  This is the amount of bytes of swappable memory in use.",
+            description="Paged-pool usage, in bytes. Swappable memory in use.",
             category="Memory",
             unit="bytes",
             extra_fields={"type": "paged_pool"},
@@ -315,7 +320,7 @@ _PROCESS_MEMORY_METRICS = [
     METRIC(  # ------------------ Peak Paged Pool ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.mem.bytes",
-            description="The peak paged-pool usage, in bytes.",
+            description="Peak paged-pool usage, in bytes.",
             category="Memory",
             unit="bytes",
             extra_fields={"type": "peak_paged_pool"},
@@ -325,8 +330,8 @@ _PROCESS_MEMORY_METRICS = [
     METRIC(  # ------------------ NonPaged Pool ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.mem.bytes",
-            description="The nonpaged pool usage, in bytes.  This is the amount of memory in use that cannot be "
-            "swapped out to disk.",
+            description="Nonpaged pool usage, in bytes. "
+            "Memory in use that cannot be swapped out to disk.",
             category="Memory",
             unit="bytes",
             extra_fields={"type": "nonpaged_pool"},
@@ -336,7 +341,7 @@ _PROCESS_MEMORY_METRICS = [
     METRIC(  # ------------------ Peak NonPaged Pool ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.mem.bytes",
-            description="The peak nonpaged pool usage, in bytes.",
+            description="Peak nonpaged pool usage, in bytes.",
             category="Memory",
             unit="bytes",
             extra_fields={"type": "peak_nonpaged_pool"},
@@ -346,7 +351,7 @@ _PROCESS_MEMORY_METRICS = [
     METRIC(  # ------------------ Pagefile ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.mem.bytes",
-            description="The current pagefile usage, in bytes.  The is the total number of bytes the system has "
+            description="Pagefile usage, in bytes. Bytes the system has "
             "committed for this running process.",
             category="Memory",
             unit="bytes",
@@ -357,7 +362,7 @@ _PROCESS_MEMORY_METRICS = [
     METRIC(  # ------------------ Peak Pagefile ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.mem.bytes",
-            description="The peak pagefile usage, in bytes.",
+            description="Peak pagefile usage, in bytes.",
             category="Memory",
             unit="bytes",
             extra_fields={"type": "peak_pagefile"},
@@ -367,7 +372,7 @@ _PROCESS_MEMORY_METRICS = [
     METRIC(  # ------------------ Resident size ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.mem.bytes",
-            description="The current resident size in bytes.  This should be the same as the working set.",
+            description="Current resident memory size, in bytes. This should be the same as the working set.",
             category="Memory",
             unit="bytes",
             extra_fields={"type": "rss"},
@@ -377,7 +382,7 @@ _PROCESS_MEMORY_METRICS = [
     METRIC(  # ------------------ Virtual memory size ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.mem.bytes",
-            description="The current virtual memory size in bytes.  This does not include shared pages.",
+            description="Virtual memory size, in bytes. Does not include shared pages.",
             category="Memory",
             unit="bytes",
             extra_fields={"type": "vms"},
@@ -396,7 +401,8 @@ _PROCESS_DISK_IO_METRICS = [
     METRIC(  # ------------------ Disk Read Operations ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.disk.ops",
-            description="The number of disk read requests issued by the process since creation time.",
+            description="Number of disk read requests. The value is cumulative since "
+            "process start; see `winproc.uptime`.",
             category="Disk",
             unit="requests",
             cumulative=True,
@@ -407,7 +413,8 @@ _PROCESS_DISK_IO_METRICS = [
     METRIC(  # ------------------ Disk Write Operations ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.disk.ops",
-            description="The number of disk write requests issued by the process since creation time.",
+            description="Number of disk write requests. The value is cumulative since "
+            "process start; see `winproc.uptime`.",
             category="Disk",
             unit="requests",
             cumulative=True,
@@ -418,7 +425,8 @@ _PROCESS_DISK_IO_METRICS = [
     METRIC(  # ------------------ Disk Read Bytes ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.disk.bytes",
-            description="The number of bytes read from disk by the process.",
+            description="Bytes read from disk. The value is cumulative since process "
+            "start; see `winproc.uptime`.",
             category="Disk",
             unit="bytes",
             cumulative=True,
@@ -429,7 +437,8 @@ _PROCESS_DISK_IO_METRICS = [
     METRIC(  # ------------------ Disk Read Bytes ----------------------------
         METRIC_CONFIG(
             metric_name="winproc.disk.bytes",
-            description="The number of bytes written to disk by the process.",
+            description="Bytes written to disk. The value is cumulative since process "
+            "start; see `winproc.uptime`.",
             category="Disk",
             unit="bytes",
             cumulative=True,
@@ -453,21 +462,24 @@ _ = [define_metric(__monitor__, **metric.config) for metric in METRICS]
 #
 # Logging / Reporting - defines the method and content in which the metrics are reported.
 #
-define_log_field(__monitor__, "monitor", "Always ``windows_process_metrics``.")
+define_log_field(__monitor__, "monitor", "Always `windows_process_metrics`.")
 define_log_field(
     __monitor__,
     "instance",
-    "The ``id`` value from the monitor configuration, e.g. ``iis``.",
+    "The `id` value, for example `tomcat`.",
 )
 define_log_field(
     __monitor__,
     "app",
-    "Same as ``instance``; provided for compatibility with the original Scalyr Agent.",
+    "Same as `instance`; created for compatibility with the original Scalyr Agent.",
 )
 define_log_field(
-    __monitor__, "metric", 'The name of a metric being measured, e.g. "winproc.cpu".'
+    __monitor__,
+    "metric",
+    'Name of the metric, for example "winproc.cpu". Some metrics have additional '
+    "fields; see the [Metrics Reference](#metrics).",
 )
-define_log_field(__monitor__, "value", "The metric value.")
+define_log_field(__monitor__, "value", "Value of the metric.")
 
 
 #
@@ -506,11 +518,73 @@ def commandline_matcher(regex, flags=re.IGNORECASE):
 
 class ProcessMonitor(ScalyrMonitor):
     # fmt: off
-    """
-    This agent monitor plugin records CPU consumption, memory usage, and other metrics for a specified process on
-    Windows system.
+    r"""
+    # Windows Process Metrics
+
+    Import CPU consumption, memory usage, and other metrics for a process, or group of processes, on a Windows server.
+
+    An [Agent Plugin](https://app.scalyr.com/help/scalyr-agent#plugins) is a component of the Scalyr Agent, enabling the collection of more data. The source code for each plugin is available on [Github](https://github.com/scalyr/scalyr-agent-2/tree/master/scalyr_agent/builtin_monitors).
+
+    You can use this plugin to monitor resource usage for a web server, database, or other application. 32-bit Windows systems are not supported.
+
+    This plugin requires installation of the python module `psutil`, typically with the command `pip install psutil`.
+
+    You can disable collection of these metrics by setting `implicit_agent_process_metrics_monitor: false` at the top level of the Agent [configuration file](/help/scalyr-agent#plugins).
+
+
+    ## Installation
+
+    1\. Install the Scalyr Agent
+
+    If you haven't already, install the [Scalyr Agent](https://app.scalyr.com/help/welcome) on the Windows server.
+
+
+    2\. Configure the Scalyr Agent to import process metrics
+
+    Open the Scalyr Agent configuration file, located at `C:\Program Files (x86)\Scalyr\config\agent.json`.
+
+    Find the `monitors: [ ... ]` section and add a `{...}` stanza with the `module` property set for windows process metrics:
+
+        monitors: [
+          {
+             module:      "scalyr_agent.builtin_monitors.windows_process_metrics",
+             id:          "tomcat",
+             commandline: "java.*tomcat6",
+          }
+        ]
+
+    The `id` property lets you identify the command whose output you are importing. It shows in the UI as a value for the `instance` field. This is especially useful if you are running multiple instances of this plugin, to import metrics from multiple processes. Add a separate `{...}` stanza for each instance, and set unique `id`s.
+
+    The `commandline` property is a [regular expression](https://app.scalyr.com/help/regex), matching on the command line output of `tasklist`, or `wmic process list`. If multiple processes match, only the first is used. The above example imports metrics for the first process whose command line output matches the regular expression `java.*tomcat6`.
+
+    You can also select a process by process identifier (PID). See [Configuration Options](#options) below.
+
+
+    3\. Save and confirm
+
+    Save the `agent.json` file. The Agent will detect changes within 30 seconds. Wait a few minutes for data to send.
+
+    You can check the [Agent Status](https://app.scalyr.com/help/scalyr-agent#agentStatus), which includes information about all running monitors.
+
+
+    4\. Configure the process metrics dashboard for each `id`
+
+    Log into Scalyr and click Dashboards > Windows Process Metrics. At the top of the dashboard you can select the `serverHost` and `process` of interest. (We map the `instance` field, explained above, to `process`).
+
+    Click `...` in upper right of the page and select "Edit JSON". Find these lines near the top of the JSON file:
+
+        // On the next line, list each "id" that you've used in a windows_process_metrics
+        // clause in the Scalyr Agent configuration file (agent.json).
+        values: [ "agent" ]
+
+    The "agent" id is used to report metrics for the Scalyr Agent. Add each `id` you created to the `values` list. For example, to add "tomcat":
+
+        values: [ "agent", "tomcat" ]
+
+    Save the file. To view all data collected by this plugin, across all servers, go to Search view and query [monitor = 'windows_process_metrics'](https://app.scalyr.com/events?filter=monitor+%3D+%27windows_process_metrics%27).
+
+    For help, contact Support.
 
-    You can use this plugin to record resource usage for a web server, database, or other application.
     """
     # fmt: on
 
